##=============================================================================
##      Parametrizations of logistic nonlinear regression model.
##
##                      Walmes Marques Zeviani,
##                          Cristiano Nunes,
##                        Larissa May De Mio,
##                    Paulo Justiniano Ribeiro Jr.
##=============================================================================

##-----------------------------------------------------------------------------
## Session definition.

require(lattice)
require(latticeExtra)
require(rpanel)
require(MASS)
require(ellipse)
require(bbmle)

rm(list=ls())
source("http://dl.dropboxusercontent.com/u/48140237/viciobox.R")
source("/home/walmes/Dropbox/CursoR/Rbras2013/funções/viciobox.R")

## require(nlstools)

##-----------------------------------------------------------------------------
## Incidence along time for a disese.

dados <- structure(
    list(
        dia=c(0L, 0L, 0L, 0L, 0L, 0L, 14L, 14L, 14L,
            14L, 14L, 14L, 24L, 24L, 24L, 24L, 24L,
            24L, 38L, 38L, 38L, 38L, 38L, 38L, 54L,
            54L, 54L, 54L, 54L, 54L, 69L, 69L, 69L,
            69L, 69L, 69L, 81L, 81L, 81L, 81L, 81L,
            81L, 92L, 92L, 92L, 92L, 92L, 92L, 106L,
            106L, 106L, 106L, 106L, 106L, 117L, 117L,
            117L, 117L, 117L, 117L, 128L, 128L, 128L,
            128L, 128L, 128L, 142L, 142L, 142L, 154L,
            154L, 166L, 166L),
        inc=c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.09, 0,
            0.13, 0, 0, 0.08, 0.18, 0.05, 0.38, 0.25, 0.13, 0.15, 0.5,
            0.14, 0.33, 0.4, 0.63, 0.16, 0.5, 0.62, 0.56, 0.73, 0.63,
            0.3, 0.89, 0.57, 0.92, 0.62, 0.75, 0.52, 1, 0.67, 1, 1,
            1, 0.53, 1, 0.85, 1, 1, 1, 0.93, 1, 1, 1, 1, 1, 0.96,
            1, 1, 1, 1, 1, 1, 1, 1, 1)), 
    .Names=c("dia", "inc"), class="data.frame") 

##-----------------------------------------------------------------------------
## Scatter plot.

p0 <-
    xyplot(inc~dia, data=dados, type=c("p","a"), jitter.x=TRUE, col=1,
           xlab="Days after first measurament",
           ylab="Disease incidence")
p0

pdf("incid.pdf", w=5, h=3)
p0
dev.off()

##-----------------------------------------------------------------------------
## These data are from a longitudinal study on 6 plants. The answer is
## the incidence is obtained by the ratio of cases/total, that is commom
## to assume binomial distribution. However, the records contained only
## the ratio calculated without information about the total. Here we
## assume valid the assumptions of ordinary regression model. Even the
## assumptions are not valid this does not avoid making comparisons
## between parameterizations.

##-----------------------------------------------------------------------------
## The logistic model is a nonlinear models "S" shaped (sigmoid) most
## known and used in data analysis, especially to represent growth
## curves/progress in all areas of knowledge. Outside the context of
## nonlinear models, the logistic model is present in the class of
## generalized linear models, used as the inverse of the link function
## for the parameter 'p' of the binomial distribution and the 'mean' of
## the Beta distribution. His extensive application contributed to the
## emergence of several parameterizations of the model. Each of which
## presents a particular interpretation for the parameters. From the
## point of view of estimation and statistical inference, the
## parameterizations have different aspects as well. We will see 4
## parameterization of the logistic model.

##-----------------------------------------------------------------------------
## Studying model paramatrizations with rpanel.

## start values
theta.a <- c(a1=73.12, a2=15.23)
theta.b <- c(b1=121.44, b2=-0.065)
theta.c <- c(c1=4.79, c2=-0.065)
theta.d <- c(d1=0.00816, d2=0.065)

##-----------------------------------------------------------------------------
## Functions for each parametrization.

## centre-scale (default R SSlogis self-start function)
f.a <- function(x, a1, a2){ 1/(1+exp((a1-x)/a2)) }

## product
f.b <- function(x, b1, b2){ 1/(1+b1*exp(b2*x)) }

## linearized form (inverse of logit link function)
f.c <- function(x, c1, c2){ 1/(1+exp(c1+c2*x)) }

## commom in fitopatology
f.d <- function(x, d1, d2){ 1/(1+(-1+1/d1)*exp(-d2*x)) }

f.apanel <- function(p){
  plot(inc~dia, dados); curve(f.a(x, p$a1, p$a2), add=TRUE)
  abline(v=p$a1+c(-1,0,1)*p$a2*10/2, lty=2)
  p
}
panel <- rp.control("centre-scale")
rp.slider(panel, a1, 20, 160, initval=80, showvalue=TRUE, action=f.apanel)
rp.slider(panel, a2, 0.1, 50, initval=5, showvalue=TRUE, action=f.apanel)

f.bpanel <- function(p){
    plot(inc~dia, dados)
    curve(f.b(x, p$b1, p$b2), add=TRUE)
    p
}
panel <- rp.control("product")
rp.slider(panel, b1, 0, 200, initval=120, showvalue=TRUE, action=f.bpanel)
rp.slider(panel, b2, 0, -3, initval=-0.06, showvalue=TRUE, action=f.bpanel)

f.cpanel <- function(p){
    plot(inc~dia, dados)
    curve(f.c(x, p$c1, p$c2), add=TRUE)
    p
}
panel <- rp.control("linearized")
rp.slider(panel, c1, 0, 20, initval=5, showvalue=TRUE, action=f.cpanel)
rp.slider(panel, c2, 0, -3, initval=-0.06, showvalue=TRUE, action=f.cpanel)

f.dpanel <- function(p){
    plot(inc~dia, dados)
    curve(f.d(x, p$d1, p$d2), add=TRUE)
    p
}
panel <- rp.control("Madden (2007)")
rp.slider(panel, d1, 0, 0.016, initval=0.008, showvalue=TRUE, action=f.dpanel)
rp.slider(panel, d2, 0, 3, initval=0.06, showvalue=TRUE, action=f.dpanel)

##-----------------------------------------------------------------------------
## It will be considered estimation by maximum likelihood using optim()
## function.

## log-likelihood funtion
ll <- function(th, y, x, model, C=1){
  ex <- do.call(model, list(x=x, th=th))
  sd <- sqrt(crossprod(y-ex)/length(x))
  ll <- sum(dnorm(y, mean=ex, sd=sd, log=TRUE))
  ll*C
}

## models written in vectorized form
f.a <- function(x, th){ 1/(1+exp((th[1]-x)/th[2])) }
f.b <- function(x, th){ 1/(1+th[1]*exp(th[2]*x)) }
f.c <- function(x, th){ 1/(1+exp(th[1]+th[2]*x)) }
f.d <- function(x, th){ 1/(1+(-1+1/th[1])*exp(-th[2]*x)) }

## data
y <- dados$inc; x <- dados$dia

## initial values list
init.list <- list(A=list(par=c(80,13), model=f.a),
                  B=list(par=c(120,-0.06), model=f.b),
                  C=list(par=c(5,-0.06), model=f.c),
                  D=list(par=c(0.008, 0.065), model=f.d))

## fixed arguments during estimation
fixed.list <- list(fn=ll, x=x, y=y, method="BFGS",
                   control=list(fnscale=-1), hessian=TRUE)

##-----------------------------------------------------------------------------
## Parameter estimation.

## estimation in series for each model
op.all <-
  lapply(init.list,
         function(i){
           op <- do.call(optim, c(i, fixed.list)); op
         })

## parameter estimates
pars <- sapply(op.all, "[[", "par"); pars

## log-likelihood
ll0 <- sapply(op.all, "[[", "value"); ll0

## covariance matrix
vcovs <- lapply(op.all, "[[", "hessian")
vcovs <- lapply(vcovs, function(i) -solve(i))

##-----------------------------------------------------------------------------
## Observed vs fitted values (equal for all models)

pdf("incidfit.pdf", w=5, h=3)
update(p0, type="p")+
    layer(panel.curve(f.a(x, th=pars[,1]), 0, 180, add=TRUE, lwd=1.4))
dev.off()

##-----------------------------------------------------------------------------
## Confidence regions based on likelihood function and quadratic
## approximation.

leng <- 100
grid.a <- expand.grid(th1=seq(67,79,l=leng),
                      th2=seq(11,20,l=leng))
grid.a$ll <- apply(grid.a, 1, ll, y=y, x=x, model=f.a)
grid.a$dev <- with(grid.a, -2*(ll-ll0["A"]))

grid.b <- expand.grid(th1=seq(30,700,l=leng),
                      th2=seq(-0.09,-0.05,l=leng))
grid.b$ll <- apply(grid.b, 1, ll, y=y, x=x, model=f.b)
grid.b$dev <- with(grid.b, -2*(ll-ll0["B"]))

grid.c <- expand.grid(th1=seq(3.5,6.7,l=leng),
                      th2=seq(-0.09,-0.05,l=leng))
grid.c$ll <- apply(grid.c, 1, ll, y=y, x=x, model=f.c)
grid.c$dev <- with(grid.c, -2*(ll-ll0["C"]))

grid.d <- expand.grid(th1=seq(0.0005,0.027,l=leng),
                      th2=seq(0.049,0.09,l=leng))
grid.d$ll <- apply(grid.d, 1, ll, y=y, x=x, model=f.d)
grid.d$dev <- with(grid.d, -2*(ll-ll0["D"]))

grid.full <- rbind(grid.a, grid.b, grid.c, grid.d)
grid.full$para <- rep(c("A","B","C","D"), each=leng^2)

expr <- c(expression(f[a](x) == 1/(1+exp((a[1]-x)/a[2]))),
          expression(f[b](x) == 1/(1+b[1]*exp(b[2]*x))),
          expression(f[c](x) == 1/(1+exp(c[1]+c[2]*x))),
          expression(f[d](x) == 1/(1+(-1+1/d[1])*exp(-d[2]*x))))

levels <- c(0.05,0.5,0.75,0.9,0.99) # cortes na deviance
qchi <- qchisq(levels, df=2)             # quantil da chi-quadrado

lay <- c(2,2)
## pdf("logisL.pdf", width=14, height=4); lay <- c(NA,1)
pdf("logisW.pdf", width=8, height=8); lay <- c(2,2)
contourplot(dev~th1+th2|para, data=grid.full,
            at=qchisq(0.95, df=2), scales="free",
            lwd=1.5,
            layout=lay,
            ## labels=list(labels=as.character(levels), cex=0.8),
            labels=FALSE,
            xlab=expression("(a,b,c,d)"[1]),
            ylab=expression("(a,b,c,d)"[2]),
            strip=strip.custom(bg="gray90", factor.levels=(expr)),
            par.settings=list(layout.heights=list(strip=1.5)),
            panel=function(...){
              panel.contourplot(...)
              i <- which.packet()
              panel.abline(v=pars[1,i], h=pars[2,i], lty=2)
              panel.points(ellipse(x=vcovs[[i]], centre=pars[,i]),
                           type="l", col=2, lty=1, lwd=1.5)
            })+
    as.layer(contourplot(dev~th1+th2|para, data=grid.full,
            at=qchi, labels=FALSE, col="gray75"))
dev.off()

##-----------------------------------------------------------------------------
## The confidence contours give the format of the log-likelihood around
## the estimates. From the point of view of estimation and inference,
## ellipsoidal and ortogonal are desirable function properties. Presence
## of ellipticity is associated with the Gaussian distribution for the
## estimators. Twisted/deformed shapes (banana shape) are high connected
## to the non-linearity of the parametrization, where it is expected
## that the measures of curvature of Bates & Watts are high. The
## orthogonality allows inference about a parameter independent of each
## other. Moreover, orthogonality favors numerical optimization
## algorithms, particularly those with type tower "shift" like Gibbs
## sampler in bayesian inference. As increases the covariance between
## parameters decreases the identifiability of the model.

##-----------------------------------------------------------------------------
## Bates & Watts curvature measures.

f.a <- deriv3(~1/(1+exp((a1-x)/a2)), c("a1","a2"),
              function(x, a1, a2) NULL )
f.b <- deriv3(~1/(1+b1*exp(b2*x)), c("b1","b2"),
              function(x, b1, b2) NULL )
f.c <- deriv3(~1/(1+exp(c1+c2*x)), c("c1","c2"),
              function(x, c1, c2) NULL )
f.d <- deriv3(~1/(1+(-1+1/d1)*exp(-d2*x)), c("d1","d2"),
              function(x, d1, d2) NULL )

f.list <- c(inc~f.a(dia, a1, a2), inc~f.b(dia, b1, b2),
            inc~f.c(dia, c1, c2), inc~f.d(dia, d1, d2))
i.list <- list(theta.a, theta.b, theta.c, theta.d)

n00 <- lapply(seq_along(f.list),
              function(i){
                nls(f.list[[i]], start=i.list[[i]], data=dados)
              })

lapply(n00, rms.curv)
lapply(n00, function(i) cov2cor(vcov(i)))
lapply(n00, biasbox)

##-----------------------------------------------------------------------------
## Inference for each parameter.

init.list <- list(A=list(par=c(80,13), model=f.a),
                  B=list(par=c(120,-0.06), model=f.b),
                  C=list(par=c(5,-0.06), model=f.c),
                  D=list(par=c(0.008, 0.065), model=f.d))

f.a <- function(x, th){  }
f.b <- function(x, th){  }
f.c <- function(x, th){ }
f.d <- function(x, th){  }

ll.a <- function(th, y, x){
  ex <- 1/(1+exp((th[1]-x)/th[2]))
  sd <- sqrt(crossprod(y-ex)/length(x))
  ll <- sum(dnorm(y, mean=ex, sd=sd, log=TRUE))
  -ll
}
parnames(ll.a) <- c("a1", "a2")
n0.a <- mle2(ll.a, start=list(a1=80,a2=13), method="BFGS", vecpar=TRUE,
           data=list(x=x, y=y))
plot(profile(n0.a))

ll.b <- function(th, y, x){
  ex <- 1/(1+th[1]*exp(th[2]*x))
  sd <- sqrt(crossprod(y-ex)/length(x))
  ll <- sum(dnorm(y, mean=ex, sd=sd, log=TRUE))
  -ll
}
parnames(ll.b) <- c("b1", "b2")
n0.b <- mle2(ll.b, start=list(b1=120, b2=-0.06), method="BFGS", vecpar=TRUE,
           data=list(x=x, y=y))
plot(profile(n0.b))

ll.c <- function(th, y, x){
  ex <- 1/(1+exp(th[1]+th[2]*x))
  sd <- sqrt(crossprod(y-ex)/length(x))
  ll <- sum(dnorm(y, mean=ex, sd=sd, log=TRUE))
  -ll
}
parnames(ll.c) <- c("c1", "c2")
n0.c <- mle2(ll.c, start=list(c1=5, c2=-0.06), method="BFGS", vecpar=TRUE,
           data=list(x=x, y=y))
plot(profile(n0.c))

ll.d <- function(th, y, x){
  ex <- 1/(1+(-1+1/th[1])*exp(-th[2]*x))
  sd <- sqrt(crossprod(y-ex)/length(x))
  ll <- sum(dnorm(y, mean=ex, sd=sd, log=TRUE))
  -ll
}
parnames(ll.d) <- c("d1", "d2")
n0.d <- mle2(ll.d, start=list(d1=0.008, d2=-0.06), method="BFGS", vecpar=TRUE,
           data=list(x=x, y=y))
plot(profile(n0.d))

## oldpal <- palette()
## palette(rep("black", length(oldpal)))

pdf("profile.pdf", w=10, h=6)
par(mfcol=c(2,4))
plot(profile(n0.a))
plot(profile(n0.b))
plot(profile(n0.c))
plot(profile(n0.d))
layout(1)
dev.off()

##-----------------------------------------------------------------------------

save.image("parametrizations.RData")

##-----------------------------------------------------------------------------
